﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include <ppltasks.h>

using namespace MobileServicesApp;
using namespace concurrency;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::Data::Json;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::ApplicationModel::DataTransfer;


// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238


MainPage::MainPage()
{
	InitializeComponent();
	items_ = ref new Vector<ToDoItems^>();	
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{	
	RegisterForShare();
	RefreshData();
}


void MainPage::ButtonRefresh_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args)
{
	RefreshData();
}


void MainPage::CheckBox_Checked_1(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args)
{
	auto cb = dynamic_cast<CheckBox^>(sender);
	ToDoItems^ item = dynamic_cast<ToDoItems^>(cb->DataContext);
	auto client = App::MobileClient;
	auto jsonObject = ref new JsonObject();		
		
	jsonObject->Insert(L"id", JsonValue::CreateNumberValue(item->ID));	
	jsonObject->Insert(L"complete", JsonValue::CreateBooleanValue(item->Complete));	

	auto table = client->GetTable("todoitem");
	task<IJsonValue^> updateTask(table->UpdateAsync(jsonObject));
	updateTask.then([this, item](IJsonValue^ V)
	{
		int x = items_->Size;
		for (int i = 0; i < x; ++i)
		{
			if (items_->GetAt(i) == item)
			{
				items_->RemoveAt(i);
			}
		}
	});
}


void MainPage::ButtonSave_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args)
{
	auto item = ref new ToDoItems();
	item->Text = TextInput->Text;
	item->Complete = false;
	item->Channel = App::CurrentChannel->Uri;
	//save data to AzureMobile
	auto client = App::MobileClient;
	auto jsonObject = ref new JsonObject();
	jsonObject->Insert(L"text", JsonValue::CreateStringValue(item->Text));
	jsonObject->Insert(L"complete", JsonValue::CreateBooleanValue(item->Complete));
	jsonObject->Insert(L"channel", JsonValue::CreateStringValue(item->Channel));
	
	auto table = client->GetTable("todoitem");
	task<IJsonValue^> insertTask(table->InsertAsync(jsonObject));
	insertTask.then([this, item](IJsonValue^ V)
	{
		auto results = V->Stringify();		
		JsonObject^ savedItem = ref new JsonObject();
		if (JsonObject::TryParse(results, &savedItem))
		{
			item->ID = savedItem->GetNamedNumber("id");
			items_->Append(item);
			ListItems->ItemsSource = items_;
		}	
	});	
}


void MainPage::RefreshData()
{
	String^ query = "Select * from todoitem";
	auto client = App::MobileClient;
	auto table = client->GetTable("todoitem");
	task<IJsonValue^> readTask(table->ReadAsync(query));
	readTask.then([this](IJsonValue^ V)
	{
		auto list = V->Stringify();
		JsonArray^ mapValue = ref new JsonArray();
		if(JsonArray::TryParse(list, &mapValue))
		{
			auto vec = mapValue->GetView();
			std::for_each(begin(vec), end(vec), [this](IJsonValue^ M)
			{
				if (M->GetObject()->GetNamedBoolean("complete") == false)
				{	
					ToDoItems^ item = ref new ToDoItems();
					item->Text = M->GetObject()->GetNamedString("text");
					item->ID = M->GetObject()->GetNamedNumber("id");
					item->Complete = M->GetObject()->GetNamedBoolean("complete");
					item->Channel = M->GetObject()->GetNamedString("channel");
					items_->Append(item);						
				}					
			});
			ListItems->ItemsSource = items_;
		}
	});	
}

void MainPage::RegisterForShare()
{
    DataTransferManager^ dataTransferManager = DataTransferManager::GetForCurrentView();
    auto dataRequestedToken = dataTransferManager->DataRequested += ref new TypedEventHandler<DataTransferManager^, DataRequestedEventArgs^>(
        this, &MainPage::ShareTextHandler);
}

void MainPage::ShareTextHandler(DataTransferManager^ sender, DataRequestedEventArgs^ e)
{
    DataRequest^ request = e->Request;
    request->Data->Properties->Title = "Sharing from Mobile Services App";
    request->Data->Properties->Description = "A demonstration of sharing the inserted item.";
	ToDoItems^ item = (ToDoItems^)(ListItems->SelectedItem);
	request->Data->SetText(item->Text);
}
