﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using CxxReadFileComponent;
// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace CSTestApp
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private CxxReadFileComponent.ReadFile readFile = new CxxReadFileComponent.ReadFile();
        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }

        private async void ReadTxtFileAsync(object sender, RoutedEventArgs e)
        {
            b1.IsEnabled = false;
            tb1.Text = "Working...";

            var asyncAction = readFile.ReadTxtFileAsync("127.txt");

            await asyncAction;
            b1.IsEnabled = true;
            tb1.Text = "Done.";
        }

        private async void ReadTxtFileWithProgressAsync(object sender, RoutedEventArgs e)
        {
            b2.IsEnabled = false;
            tb2.Text = "Working...";

            var asyncAction = readFile.ReadTxtFileWithProgressAsync("127.txt");
            asyncAction.Progress = new AsyncActionProgressHandler<double>((action, progress) =>
            {
                pb1.Value = progress;
            });

            await asyncAction;

            tb2.Text = "Done";
            b2.IsEnabled = true;
        }

        private async void GetByteCountAsync(object sender, RoutedEventArgs e)
        {
            b3.IsEnabled = false;
            tb3.Text = "Working...";

            var asyncOperation = readFile.GetByteCountAsync("127.txt");

            await asyncOperation;

            tb3.Text = "Read " + asyncOperation.GetResults().ToString() + " bytes of data";
            b3.IsEnabled = true;

        }

        private async void GetByteCountWithProgressAsync(object sender, RoutedEventArgs e)
        {
            b4.IsEnabled = false;
            tb4.Text = "Working...";

            var asyncOperation = readFile.GetByteCountWithProgressAsync("127.txt");
            asyncOperation.Progress = new AsyncOperationProgressHandler<int, double>((operation, progress) =>
            {
                pb4.Value = progress;
            });

            await asyncOperation;

            tb4.Text = "Read " + asyncOperation.GetResults().ToString() + " bytes of data";
            b4.IsEnabled = true;
        }

        private async void GetByteCountHandleErrors(object sender, RoutedEventArgs e)
        {
            b5.IsEnabled = false;
            tb5.Text = "Working...";

            var asyncOperation = readFile.GetByteCountWithProgressAsync("127.txt");
            asyncOperation.Progress = new AsyncOperationProgressHandler<int, double>((operation, progress) =>
            {
                pb5.Value = progress;
            });

            try
            {
                await asyncOperation;
                tb5.Text = "Read " + asyncOperation.GetResults().ToString() + " bytes of data";
            }
            catch (ArgumentException ex)
            {
                tb5.Text = "ERROR: " + ex.Message;
            }

            b5.IsEnabled = true;
        }


        private IAsyncOperationWithProgress<int, double> asyncCancelableOperation;

        private async void GetByteCountCancellation(object sender, RoutedEventArgs e)
        {
            b6.IsEnabled = false;
            cancelButton.IsEnabled = true;
            tb6.Text = "Working...";

            asyncCancelableOperation = readFile.GetByteCountWithProgressAsync("127.txt");
            asyncCancelableOperation.Progress = new AsyncOperationProgressHandler<int, double>((operation, progress) =>
            {
                pb6.Value = progress;
            });

            try
            {
                await asyncCancelableOperation;
                tb6.Text = "Read " + asyncCancelableOperation.GetResults().ToString() + " bytes of data";
            }
            catch (System.Threading.Tasks.TaskCanceledException)
            {
                tb6.Text = "Operation canceled";
            }

            b6.IsEnabled = true;
            cancelButton.IsEnabled = false;
        }

        private void cancelGetPrimes(object sender, RoutedEventArgs e)
        {
            cancelButton.IsEnabled = false;
            asyncCancelableOperation.Cancel();
        }

    }
}
