﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include "Buttons.xaml.h"
#include "ComboBox-Intro.xaml.h"
#include "misc-controls.xaml.h"
#include "progress-intro.xaml.h"
#include "styling-intro.xaml.h"
#include "slider-intro.xaml.h"
#include "templating-intro.xaml.h"
#include "text-input.xaml.h"
#include "vsm-intro.xaml.h"
#include "Utils\SuspensionManager.h"

using namespace BasicControls;
using namespace BasicControls::Common;

using namespace Platform;
using namespace Platform::Collections;
using namespace Platform::Details;	
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::ViewManagement;
// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238
MainPage^ MainPage::_current = nullptr;

MainPage::MainPage()
{	
	InitializeComponent();
	_current = this;
	_hiddenFrame = ref new Windows::UI::Xaml::Controls::Frame();
	_hiddenFrame->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	ContentRoot->Children->Append(_hiddenFrame);

	FEATURE_NAME = "Basic Controls Sample";
	Scenarios->SelectionChanged += ref new SelectionChangedEventHandler(this, &MainPage::Scenarios_SelectionChanged);
	
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	BuildScenarios();
	PopulateScenarios();	
	InvalidateViewState();
}

void MainPage::InvalidateViewState()
{
	if (ApplicationView::Value == ApplicationViewState::Snapped)
	{
		Grid::SetRow(DescriptionText, 3);
        Grid::SetColumn(DescriptionText, 0);

        Grid::SetRow(InputSection, 4);
        Grid::SetColumn(InputSection, 0);

	}
	else
	{
		Grid::SetRow(DescriptionText, 1);
        Grid::SetColumn(DescriptionText, 1);

        Grid::SetRow(InputSection, 2);
        Grid::SetColumn(InputSection, 1);

	}
	VisualStateManager::GoToState((Control^)InputSection, L"Input" + ApplicationView::Value.ToString(), false);
	VisualStateManager::GoToState((Control^)OutputSection, L"Output" + ApplicationView::Value.ToString(), false);
}

void MainPage::PopulateScenarios()
{
	Vector<Object^>^ ScenarioList = ref new Vector<Object^>();
	int i = 0;
	
	auto size = _scenario.size();
	for (UINT32 x = 0; x < size; ++x)
	{
		auto item = ref new ListBoxItem();
		_scenario[x]->Title = (++i).ToString() + ") " + _scenario[x]->Title;
		item->Content = _scenario[x]->ToString();
		//using the DataContext property here since Content does not work when
		// fetching back the Scenario object...
		item->DataContext = _scenario[x];
		ScenarioList->Append(item);
	}
	Scenarios->ItemsSource = ScenarioList;

	int startingScenarioIndex = -1;
	if (SuspensionManager::SessionState->HasKey("SelectedScenarioIndex"))
    {
		auto index = SuspensionManager::SessionState->Lookup("SelectedScenarioIndex");		
		int selectedScenarioIndex = static_cast<int>(index);        
        startingScenarioIndex = selectedScenarioIndex;
    }
    Scenarios->SelectedIndex = startingScenarioIndex != -1 ? startingScenarioIndex : 0;
    Scenarios->ScrollIntoView(Scenarios->SelectedItem);
}

void MainPage::LoadScenario(Windows::UI::Xaml::Interop::TypeName classType)
{
	_hiddenFrame->Navigate(classType, this);
	Page^ hidden = dynamic_cast<Page^>(_hiddenFrame->Content);

	UIElement^ input = dynamic_cast<UIElement^>(hidden->FindName("Input"));
	UIElement^ output = dynamic_cast<UIElement^>(hidden->FindName("Output"));
	Panel^ panel = dynamic_cast<Panel^>(hidden->FindName("LayoutRoot"));
	if (panel != nullptr)
	{
		auto size = panel->Children->Size;
		for (UINT32 i = 0; i < size; ++i)
		{
			if (panel->Children->GetAt(i) == input)
			{
				panel->Children->RemoveAt(i);
			}
			if (panel->Children->GetAt(i) == output)
			{
				panel->Children->RemoveAt(i);
			}
		}
		
		InputSection->Content = input;
		OutputSection->Content = output;
	}

}

void MainPage::Scenarios_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ args)
{
	if (Scenarios->SelectedItem != nullptr)
	{
		ListBoxItem^ item = dynamic_cast<ListBoxItem^>(Scenarios->SelectedItem);
		SuspensionManager::SessionState->Insert("SelectedScenarioIndex", Scenarios->SelectedItem);
		auto scenario = (Scenario^)item->DataContext;
		LoadScenario(scenario->ClassType);
	}
}


void MainPage::BuildScenarios()
{
	//_scenario = ref new Vector<BasicControls::Scenario^>();
	
	auto s1 = ref new Scenario();
	s1->Title = "Slider - Introduction";	
	s1->ClassType = TypeName(SliderIntro::typeid);
	_scenario.push_back(s1);
	
	auto s2 = ref new Scenario();
	s2->Title = "Progress - Introduction";	
	s2->ClassType = TypeName(ProgressIntro::typeid);
	_scenario.push_back(s2);
	
	auto s3 = ref new Scenario();
	s3->Title = "Buttons";	
	s3->ClassType = TypeName(Buttons::typeid);
	_scenario.push_back(s3);

	auto s4 = ref new Scenario();
	s4->Title = "Text input";
	s4->ClassType = TypeName(TextInput::typeid);
	_scenario.push_back(s4);
	
	auto s5 = ref new Scenario();
	s5->Title = "Combo/List boxes";
	s5->ClassType = TypeName(ComboboxIntro::typeid);
	_scenario.push_back(s5);
	
	auto s6 = ref new Scenario();
	s6->Title = "Miscellaneous controls";
	s6->ClassType = TypeName(MiscControls::typeid);
	_scenario.push_back(s6);
	
	auto s7 = ref new Scenario();
	s7->Title = "Styling a control";
	s7->ClassType = TypeName(StylingIntro::typeid);
	_scenario.push_back(s7);
	
	auto s8 = ref new Scenario();
	s8->Title = "Templating a control";	
	s8->ClassType = TypeName(TemplatingIntro::typeid);
	_scenario.push_back(s8);
	
	auto s9 = ref new Scenario();
	s9->Title = "Visual State Manager";	
	s9->ClassType = TypeName(VisualStateManagerIntro::typeid);
	_scenario.push_back(s9);
	
}