﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include "ColorsFlyout.xaml.h"
#include "SizeFlyout.xaml.h"

using namespace DrawIt;

using namespace concurrency;
using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::ApplicationModel;
using namespace Windows::ApplicationModel::Activation;
using namespace Windows::ApplicationModel::Core;
using namespace Windows::ApplicationModel::DataTransfer;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::System;
using namespace Windows::UI::Core;
using namespace Windows::UI::Popups;
using namespace Windows::UI::ViewManagement;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Microsoft::WRL;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage()
{
    InitializeComponent();

    m_renderer = ref new ShapeRenderer();
    m_renderer->Initialize(Window::Current->CoreWindow, this->DXSwapChainPanel, DisplayProperties::LogicalDpi);
    m_renderer->Render();    

    DisplayProperties::LogicalDpiChanged +=
        ref new DisplayPropertiesEventHandler(this, &MainPage::OnLogicalDpiChanged);

    Window::Current->CoreWindow->SizeChanged +=
        ref new TypedEventHandler<CoreWindow^, WindowSizeChangedEventArgs^>(this, &MainPage::OnWindowSizeChanged);

    this->PointerPressed +=
        ref new PointerEventHandler(this, &MainPage::OnPointerPressed);

    this->PointerMoved +=
        ref new PointerEventHandler(this, &MainPage::OnPointerMoved);

    this->PointerReleased +=
        ref new PointerEventHandler(this, &MainPage::OnPointerReleased);

    this->ManipulationMode =
        ManipulationModes::TranslateX |
        ManipulationModes::TranslateY |
        ManipulationModes::Scale;

    DataTransferManager::GetForCurrentView()->DataRequested +=
        ref new TypedEventHandler<DataTransferManager^, DataRequestedEventArgs^>(m_renderer, &ShapeRenderer::OnDataRequested);

    VisualStateManager::GoToState(this, "ClearState", true);
    m_atStartScreen = false;
}

void MainPage::AddColorsClicked(Object^ sender, RoutedEventArgs^ e)
{
    auto genTrans = AddColorsButtonBorder->TransformToVisual(this);
    auto point = genTrans->TransformPoint(Point(0, 0));

    ColorsPopup->VerticalOffset = point.Y - ColorsFlyout->Height - 10;
    ColorsPopup->HorizontalOffset = point.X;
    ColorsPopup->IsOpen = true;
}

namespace
{
    inline void SetColor(ShapeRenderer^ renderer, Windows::UI::Color color)
    {
        renderer->OnChangeDrawingAttributes(color, Windows::Foundation::Size(0.0f, 0.0f));
        renderer->Render();
    }
}

void MainPage::RedColorClicked()
{
    SetColor(m_renderer, Windows::UI::Colors::Red);
}

void MainPage::GreenColorClicked()
{
    SetColor(m_renderer, Windows::UI::Colors::Green);
}

void MainPage::SaveClicked(Object^ sender, RoutedEventArgs^ e)
{
    auto savePicker = ref new FileSavePicker();
    auto pngExtensions = ref new Vector<String^>();
    pngExtensions->Append(".png");
    savePicker->FileTypeChoices->Insert("PNG file", pngExtensions);
    auto jpgExtensions = ref new Vector<String^>();
    jpgExtensions->Append(".jpg");
    savePicker->FileTypeChoices->Insert("JPEG file", jpgExtensions);
    auto bmpExtensions = ref new Vector<String^>();
    bmpExtensions->Append(".bmp");
    savePicker->FileTypeChoices->Insert("BMP file", bmpExtensions);
    savePicker->DefaultFileExtension = ".png";
    savePicker->SuggestedFileName = "MyDrawing";
    savePicker->SuggestedStartLocation = Pickers::PickerLocationId::PicturesLibrary;

    auto wicFormat = std::make_shared<GUID>(GUID_ContainerFormatPng);

    create_task(savePicker->PickSaveFileAsync()).then([=](StorageFile^ file)
    {
        if (file == nullptr)
        {
            cancel_current_task();
        }

        // Select the appropriate format based on the user-selected file extension.
        if (file->FileType == ".bmp")
        {
            *wicFormat = GUID_ContainerFormatBmp;
        }
        else if (file->FileType == ".jpg")
        {
            *wicFormat = GUID_ContainerFormatJpeg;
        }
        return file->OpenAsync(FileAccessMode::ReadWrite);
    }).then([=](Streams::IRandomAccessStream^ randomAccessStream)
    {
        m_renderer->SaveCanvas(randomAccessStream, *wicFormat);
    });
}

void MainPage::ShareClicked(Object^ sender, RoutedEventArgs^ e)
{
    DataTransferManager::ShowShareUI();
}

void MainPage::OnLogicalDpiChanged(Object^ sender)
{
    m_renderer->SetDpi(DisplayProperties::LogicalDpi);
    m_renderer->Render();
}

void MainPage::OnWindowSizeChanged(CoreWindow^ /* sender */, WindowSizeChangedEventArgs^ args)
{
    if (ApplicationView::Value == ApplicationViewState::Snapped)
    {
        VisualStateManager::GoToState(this, "SnappedState", true);
    }
    else
    {
        if (m_atStartScreen)
        {
            VisualStateManager::GoToState(this, "StartingState", true);
        }
        else
        {
            VisualStateManager::GoToState(this, "ClearState", true);
        }
    }

    m_renderer->UpdateForWindowSizeChange();
    m_renderer->Render();
    m_renderer->Present();
}

void MainPage::OnPointerPressed(Object^ sender, PointerRoutedEventArgs^ args)
{
    m_renderer->OnPointerPressed(sender, args);
}

void MainPage::OnPointerReleased(Object^ sender, PointerRoutedEventArgs^ args)
{
    m_renderer->OnPointerReleased(sender, args);
}

void MainPage::OnPointerMoved(Object^ sender, PointerRoutedEventArgs^ args)
{
    m_renderer->OnPointerMoved(sender, args);
}

void MainPage::AddSizeChangeClicked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    auto genTrans = ChangeSizeButtonBorder->TransformToVisual(this);
    auto point = genTrans->TransformPoint(Point(0, 0));

    SizePopup->VerticalOffset = point.Y - SizeFlyout->Height - 10;
    SizePopup->HorizontalOffset = point.X;
    SizePopup->IsOpen = true;
}

namespace
{
    inline void SetPenSize(ShapeRenderer^ renderer, float size)
    {
        auto attributes = renderer->DrawingAttributes;
        renderer->OnChangeDrawingAttributes(attributes->Color, Windows::Foundation::Size(size, size));
        renderer->Render();
    }
}

void MainPage::SixPointButtonClicked()
{
    SetPenSize(m_renderer, 6);
}

void MainPage::EightPointButtonClicked()
{
    SetPenSize(m_renderer, 8);
}

void MainPage::TenPointButtonClicked()
{
    SetPenSize(m_renderer, 10);
}
