////// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
////// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
////// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
////// PARTICULAR PURPOSE.
//////
////// Copyright (c) Microsoft Corporation. All rights reserved
//
#pragma once
//
#include "pch.h"

#ifndef DXHELPERS_H_GUARD
#define DXHELPERS_H_GUARD

namespace DXHelpers
{
    inline void ThrowIfFailed(HRESULT hr)
    {
        if (FAILED(hr))
        {
			RaiseException(EXCEPTION_BREAKPOINT, NULL, 0, nullptr);
        }
    }

	inline D2D1::ColorF ConvertToColorF(Windows::UI::Color* color)
	{
		return D2D1::ColorF((float)(color->R), (float)color->G, (float)color->B, 255.0f / (float)color->A);
	}

	inline D2D1_RECT_F ConvertRect(Windows::Foundation::Rect* source)
	{
		return D2D1::RectF(source->X, source->Y, source->X + source->Width, source->Y + source->Height);
	}

	inline IInspectable* GetIInspectable(Platform::Object^ object)
    {
		// workaround for casting due to C++ compiler error
		//return *reinterpret_cast<IInspectable**>(&object); 

		return (IInspectable*) reinterpret_cast<IInspectable*>(object);
    }

	template <typename T>
	inline void SafeRelease(T *&p)
	{
		if (NULL != p)
		{
			p->Release();
			p = NULL;
		}
	}
}

#endif //DXHELPERS_H_GUARD