////// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
////// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
////// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
////// PARTICULAR PURPOSE.
//////
////// Copyright (c) Microsoft Corporation. All rights reserved
//
#pragma once
//
#include "pch.h"

#include <ppl.h>
#include <ppltasks.h>

#ifndef DXHELPERS_H_GUARD
#define DXHELPERS_H_GUARD

namespace DXHelpers
{
    inline void ThrowIfFailed(HRESULT hr)
    {
        if (FAILED(hr))
        {
            //__cli_WinRTraiseException(hr);
			RaiseException(EXCEPTION_BREAKPOINT, NULL, 0, nullptr);
        }
    }

	inline D2D1::ColorF ConvertToColorF(Windows::UI::Color* color)
	{
		return D2D1::ColorF((float)(color->R), (float)color->G, (float)color->B, 255.0f / (float)color->A);
	}

	inline D2D1_RECT_F ConvertRect(Windows::Foundation::Rect* source)
	{
		return D2D1::RectF(source->X, source->Y, source->X + source->Width, source->Y + source->Height);
	}

	inline IInspectable* GetIInspectable(Platform::Object^ object)
    {
		// workaround for casting due to compiler error
		//return *reinterpret_cast<IInspectable**>(&object); 

		return (IInspectable*) reinterpret_cast<IInspectable*>(object);
    }

	// Function that reads from a binary file asynchronously.
    inline Concurrency::task<Platform::Array<byte>^> ReadDataAsync(Platform::String^ filename)
    {
        using namespace Windows::Storage;
        using namespace Concurrency;
 
        auto folder = Windows::ApplicationModel::Package::Current->InstalledLocation;
 
        return create_task(folder->GetFileAsync(filename)).then([] (StorageFile^ file) 
        {
            return FileIO::ReadBufferAsync(file);
        }).then([] (Streams::IBuffer^ fileBuffer) -> Platform::Array<byte>^ 
        {
            auto fileData = ref new Platform::Array<byte>(fileBuffer->Length);
            Streams::DataReader::FromBuffer(fileBuffer)->ReadBytes(fileData);
            return fileData;
        });
    }

}

#endif //DXHELPERS_H_GUARD