#include "pch.h"
#include "DirectXPanelBase.h"
#include "DirectXHelpers.h"

using namespace Microsoft::WRL;
using namespace Windows::Foundation;
using namespace Windows::UI::Xaml::Controls;
using namespace DXHelpers;
using namespace D2D1;

//DirectXPanelBase::DirectXPanelBase()
//{
//}
//
//
//DirectXPanelBase::~DirectXPanelBase()
//{
//}

void DirectXPanelBase::Initialize(SwapChainBackgroundPanel^ panel, Windows::Foundation::Rect bounds, float dpi, bool enableD2D)
{
	UNREFERENCED_PARAMETER(enableD2D);

	m_bounds = bounds;
	m_dpi = dpi;
	m_panel = panel;

	CreateDeviceIndependentResources(panel);
    CreateDeviceResources();
    CreateSizeDependentResources();
}

void DirectXPanelBase::CreateDeviceIndependentResources(SwapChainBackgroundPanel^ panel)
{
	IInspectable* panelInspectable = GetIInspectable(panel);

	ThrowIfFailed(
	panelInspectable->QueryInterface(__uuidof(ISwapChainBackgroundPanelNative), (void **)&m_panelNative)
	);
}

void DirectXPanelBase::CreateDeviceResources()
{	
	UINT flags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;	

#if defined(_DEBUG)    
	flags |= D3D11_CREATE_DEVICE_DEBUG;
#endif
	
    std::array<D3D_FEATURE_LEVEL, 7> featureLevels = 
    {
        D3D_FEATURE_LEVEL_11_1,
        D3D_FEATURE_LEVEL_11_0,
        D3D_FEATURE_LEVEL_10_1,
        D3D_FEATURE_LEVEL_10_0,
        D3D_FEATURE_LEVEL_9_3,
        D3D_FEATURE_LEVEL_9_2,
        D3D_FEATURE_LEVEL_9_1
    };

	// create D3D device
	ComPtr<ID3D11Device> device;
	ComPtr<ID3D11DeviceContext> context;
	ThrowIfFailed(
	D3D11CreateDevice(
		nullptr,
		D3D_DRIVER_TYPE_HARDWARE,
		nullptr,
		flags,
		featureLevels.data(),
		UINT(featureLevels.size()),
		D3D11_SDK_VERSION,
		&device,
		&m_featureLevel,
		&context
		)
	);	

	// get D3D device, context
	ThrowIfFailed(
	device.As(&m_d3dDevice)
	);
	ThrowIfFailed(
	context.As(&m_d3dContext)
	);

	if (m_d2dIsEnabled)
	{
		// QI for DXGI device
		ComPtr<IDXGIDevice> dxgiDevice;
		ThrowIfFailed(
		m_d3dDevice.As(&dxgiDevice)
		);
	
		// get D2D device, context
		ThrowIfFailed(
		D2D1CreateDevice(dxgiDevice.Get(), NULL, &m_d2dDevice)
		);
		ThrowIfFailed(
		m_d2dDevice->CreateDeviceContext(D2D1_DEVICE_CONTEXT_OPTIONS_NONE, &m_d2dContext)
		);
	
		m_d2dContext->SetDpi(m_dpi, m_dpi);		
	}

	m_swapChain = nullptr;
}

void DirectXPanelBase::CreateSizeDependentResources()
{
    // if swap chain already exists, resize it
    if(m_swapChain != nullptr)
    {
        ThrowIfFailed(
        m_swapChain->ResizeBuffers(2, 0, 0, DXGI_FORMAT_B8G8R8A8_UNORM, 0)
        );
    }    
    else // create a new swap chain
    {
		// TODO should this be double-buffered and trying to use flip?

        // descriptor
        DXGI_SWAP_CHAIN_DESC1 swapChainDesc = {0};
        swapChainDesc.Width = static_cast<UINT>(m_bounds.Width);
        swapChainDesc.Height = static_cast<UINT>(m_bounds.Height);
        swapChainDesc.Format = DXGI_FORMAT_B8G8R8A8_UNORM;           // this is the most common swapchain format
        swapChainDesc.Stereo = false; 
        swapChainDesc.SampleDesc.Count = 1;                          // don't use multi-sampling
        swapChainDesc.SampleDesc.Quality = 0;
        swapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
        swapChainDesc.BufferCount = 2;
        swapChainDesc.Scaling = DXGI_SCALING_STRETCH;
        swapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL; // we recommend using this swap effect for all applications
        swapChainDesc.Flags = 0;
        
		
		// QI for DXGI device
		ComPtr<IDXGIDevice> dxgiDevice;
		ThrowIfFailed(
		m_d3dDevice.As(&dxgiDevice)
		);

        // get adapter
        ComPtr<IDXGIAdapter> dxgiAdapter;
        ThrowIfFailed(
        dxgiDevice->GetAdapter(&dxgiAdapter)
        );

        // get factory
        ComPtr<IDXGIFactory2> dxgiFactory;
        ThrowIfFailed(
		dxgiAdapter->GetParent(__uuidof(IDXGIFactory2), &dxgiFactory)
        );

		// TODO correct swap chain type?
		
        // create swap chain
		ThrowIfFailed(
        dxgiFactory->CreateSwapChainForComposition(
            m_d3dDevice.Get(),            
            &swapChainDesc,
            nullptr,		// allow on any display 
            &m_swapChain
            )
        );
		
		ThrowIfFailed(
		m_panelNative->SetSwapChain(m_swapChain.Get())
		);
    }

    // get backbuffer
    ComPtr<ID3D11Texture2D> backBuffer;
    ThrowIfFailed(
    m_swapChain->GetBuffer(
        0,
        __uuidof(ID3D11Texture2D),
        &backBuffer
        )
    );

    // create RTV to use on bind
    ThrowIfFailed(
    m_d3dDevice->CreateRenderTargetView(
        backBuffer.Get(),
        nullptr,
        &m_renderTargetView
        )
    );

    // cache rendertarget dimensions
    D3D11_TEXTURE2D_DESC backBufferDesc = {0};
    backBuffer->GetDesc(&backBufferDesc);
    m_renderTargetSize.Width  = static_cast<float>(backBufferDesc.Width);
    m_renderTargetSize.Height = static_cast<float>(backBufferDesc.Height);

    // depthstencil
    CD3D11_TEXTURE2D_DESC depthStencilDesc(
        DXGI_FORMAT_D24_UNORM_S8_UINT, 
        backBufferDesc.Width,
        backBufferDesc.Height,
        1,
        1,
        D3D11_BIND_DEPTH_STENCIL
		);

    ComPtr<ID3D11Texture2D> depthStencil;
    ThrowIfFailed(
    m_d3dDevice->CreateTexture2D(
        &depthStencilDesc,
        nullptr,
        &depthStencil
        )
    );
    
	CD3D11_DEPTH_STENCIL_VIEW_DESC desc(D3D11_DSV_DIMENSION_TEXTURE2D);
    
	ThrowIfFailed(
    m_d3dDevice->CreateDepthStencilView(
        depthStencil.Get(),
        &desc,
        &m_depthStencilView
        )
    );

    // viewport
    CD3D11_VIEWPORT viewport(
        0.0f,
        0.0f,
        static_cast<float>(backBufferDesc.Width),
        static_cast<float>(backBufferDesc.Height)
        );

    // set viewport
    m_d3dContext->RSSetViewports(1, &viewport);

	if (m_d2dIsEnabled)
	{
		// set up D2D render target bitmap, link to swap chain
		D2D1_BITMAP_PROPERTIES1 bitmapProperties = 
			BitmapProperties1(
				D2D1_BITMAP_OPTIONS_TARGET | D2D1_BITMAP_OPTIONS_CANNOT_DRAW,
				PixelFormat(DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED),
				m_dpi,
				m_dpi
				);

		// get backbuffer as DXGI surface
		ComPtr<IDXGISurface> dxgiBackBuffer;
		ThrowIfFailed(
		m_swapChain->GetBuffer(
			0,
			__uuidof(IDXGISurface),
			&dxgiBackBuffer
			)
		);

		// set D2D render target
		ThrowIfFailed(
		m_d2dContext->CreateBitmapFromDxgiSurface(
			dxgiBackBuffer.Get(),
			&bitmapProperties,
			&m_d2dTargetBitmap
			)
		);
    
		m_d2dContext->SetTarget(m_d2dTargetBitmap.Get());
		m_d2dContext->SetTextAntialiasMode(D2D1_TEXT_ANTIALIAS_MODE_GRAYSCALE);
	}
}

void DirectXPanelBase::Resize(Windows::Foundation::Rect newBounds)
{
	if (m_bounds.Width != newBounds.Width || m_bounds.Height != newBounds.Height)
	{
		m_bounds = newBounds;        
		OnSizeChanged();
	}	
}

void DirectXPanelBase::SetDpi(float dpi)
{
    if (m_dpi != dpi)
	{
		m_dpi = dpi;        
		OnSizeChanged();
	}
}

void DirectXPanelBase::OnSizeChanged()
{
	if (m_d2dIsEnabled)
	{
		m_d2dContext->SetTarget(nullptr);
		m_d2dTargetBitmap = nullptr;
	}

    m_renderTargetView = nullptr;
    m_depthStencilView = nullptr;	

	CreateSizeDependentResources();
}

void DirectXPanelBase::Present()
{ 
    HRESULT hr = m_swapChain->Present(1, 0);
 
    if (hr == DXGI_ERROR_DEVICE_REMOVED || hr == DXGI_ERROR_DEVICE_RESET)
    {
        Initialize(m_panel, m_bounds, m_dpi, m_d2dIsEnabled);
    }
    else
    {
        ThrowIfFailed(hr);
    }
}

