#include "pch.h"
#include "TriangleRenderer.h"

using namespace Microsoft::WRL;

using namespace Windows::Foundation;
using namespace Windows::UI::Core;
using namespace Windows::UI::Xaml::Media::Imaging;
using namespace Windows::UI::Xaml::Controls;

using namespace DXHelpers;


// set up a triangle
std::array<float2, 3> m_triangleVertices = 
    {
        float2(-0.5f, -0.5f),
        float2( 0.0f,  0.5f),
        float2( 0.5f, -0.5f)
    };

std::array<unsigned short, 3> m_triangleIndices = {0, 1, 2};


TriangleRenderer::TriangleRenderer(SwapChainBackgroundPanel^ panel, Windows::Foundation::Rect bounds, float dpi)
{
	Initialize(panel, bounds, dpi, false);
}

TriangleRenderer::~TriangleRenderer()
{
}

void TriangleRenderer::Render()
{	
	// set render target
	m_d3dContext->OMSetRenderTargets(1, m_renderTargetView.GetAddressOf(), nullptr); // no depth stencil

	// set vbuffer input layout
	m_d3dContext->IASetInputLayout(m_inputLayout.Get());

	// set vbuffer
	UINT stride = sizeof(float2);
    UINT vBufferOffset = 0;    
	m_d3dContext->IASetVertexBuffers(0, 1, m_vertexBuffer.GetAddressOf(), &stride, &vBufferOffset);

	// set index buffer
    m_d3dContext->IASetIndexBuffer(m_indexBuffer.Get(), DXGI_FORMAT_R16_UINT, 0);

	// set topology to triangle list
    m_d3dContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

    // set shader stage states
    m_d3dContext->VSSetShader(m_vertexShader.Get(), nullptr, 0);
    m_d3dContext->PSSetShader(m_pixelShader.Get(), nullptr, 0);
	
	// draw
	m_d3dContext->DrawIndexed(UINT(m_triangleIndices.size()), 0, 0);		
}

void TriangleRenderer::CreateDeviceResources()
{
	DirectXPanelBase::CreateDeviceResources();

	// load vshader
	DXHelpers::ReadDataAsync("SimpleVertexShader.cso").then([this](Platform::Array<byte>^ fileData) {
        m_d3dDevice->CreateVertexShader(fileData->Data, fileData->Length, nullptr, &m_vertexShader);
        const D3D11_INPUT_ELEMENT_DESC basicVertexLayoutDesc[] = {{ "POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0,  0, D3D11_INPUT_PER_VERTEX_DATA, 0 }};
        m_d3dDevice->CreateInputLayout(basicVertexLayoutDesc, ARRAYSIZE(basicVertexLayoutDesc), fileData->Data, fileData->Length, &m_inputLayout);
 
    });

	// load pshader
	DXHelpers::ReadDataAsync("SimplePixelShader.cso").then([this](Platform::Array<byte>^ fileData){
        m_d3dDevice->CreatePixelShader(fileData->Data, fileData->Length, nullptr, &m_pixelShader);    
    });


	// create static vertex buffer
    D3D11_BUFFER_DESC vertexBufferDesc = {0};
	vertexBufferDesc.ByteWidth = sizeof(float2) * UINT(m_triangleVertices.size());
    vertexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
    vertexBufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
    vertexBufferDesc.CPUAccessFlags = 0;
    vertexBufferDesc.MiscFlags = 0;
    vertexBufferDesc.StructureByteStride = 0;

    D3D11_SUBRESOURCE_DATA vertexBufferData;
	vertexBufferData.pSysMem = m_triangleVertices.data();
    vertexBufferData.SysMemPitch = 0;
    vertexBufferData.SysMemSlicePitch = 0;
     
    m_d3dDevice->CreateBuffer(&vertexBufferDesc, &vertexBufferData, &m_vertexBuffer);
	
	// create index buffer
    D3D11_BUFFER_DESC indexBufferDesc;
	indexBufferDesc.ByteWidth = sizeof(unsigned short) * UINT(m_triangleIndices.size()); // sizeof(triangleIndices)
    indexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
    indexBufferDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
    indexBufferDesc.CPUAccessFlags = 0;
    indexBufferDesc.MiscFlags = 0;
    indexBufferDesc.StructureByteStride = 0;

    D3D11_SUBRESOURCE_DATA indexBufferData;
	indexBufferData.pSysMem = m_triangleIndices.data();
    indexBufferData.SysMemPitch = 0;
    indexBufferData.SysMemSlicePitch = 0;
		
    m_d3dDevice->CreateBuffer(&indexBufferDesc, &indexBufferData, &m_indexBuffer);
    
}
