//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

#pragma once
#include "pch.h"
#include <ppl.h>
#include <ppltasks.h>


// This header defines helper utilities to make DirectX APIs work with exceptions.
namespace DX
{
    inline void ThrowIfFailed(HRESULT hr)
    {
        if (FAILED(hr))
        {
            // Set a breakpoint on this line to catch DX API errors.
            throw Platform::Exception::CreateException(hr);
        }
    }

	inline D2D1_COLOR_F ConvertToColorF(Windows::UI::Color color)
	{
		return D2D1::ColorF(color.R / 255.0f, color.G / 255.0f, color.B / 255.0f, color.A / 255.0f);
	}

	// Function that reads from a binary file asynchronously.
    inline Concurrency::task<Platform::Array<byte>^> ReadDataAsync(Platform::String^ filename)
    {
        using namespace Windows::Storage;
        using namespace Concurrency;
 
        auto folder = Windows::ApplicationModel::Package::Current->InstalledLocation;
 
        return create_task(folder->GetFileAsync(filename)).then([] (StorageFile^ file) 
        {
            return FileIO::ReadBufferAsync(file);
        }).then([] (Streams::IBuffer^ fileBuffer) -> Platform::Array<byte>^ 
        {
            auto fileData = ref new Platform::Array<byte>(fileBuffer->Length);
            Streams::DataReader::FromBuffer(fileBuffer)->ReadBytes(fileData);
            return fileData;
        });
    }

	inline Concurrency::task<Platform::Array<byte>^> ReadPictureDataAsync(Windows::Storage::StorageFile^ file)
	{
		using namespace Windows::Storage;
        using namespace Concurrency;
 
		return create_task(FileIO::ReadBufferAsync(file)).then([] (Streams::IBuffer^ fileBuffer) -> Platform::Array<byte>^ 
        {
            auto fileData = ref new Platform::Array<byte>(fileBuffer->Length);
            Streams::DataReader::FromBuffer(fileBuffer)->ReadBytes(fileData);
            return fileData;
        });
	}


}
