﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include "EffectsFlyout.xaml.h"

using namespace ImageEffects;

using namespace concurrency;
using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::ApplicationModel;
using namespace Windows::ApplicationModel::Activation;
using namespace Windows::ApplicationModel::Core;
using namespace Windows::ApplicationModel::DataTransfer;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::System;
using namespace Windows::UI::Core;
using namespace Windows::UI::Popups;
using namespace Windows::UI::ViewManagement;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Microsoft::WRL;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage() : m_renderNeeded(true)
{		
	InitializeComponent();

	m_effects = ref new PictureEffects();
    m_effects->Initialize(Window::Current->CoreWindow, this->DXSwapChainPanel, DisplayProperties::LogicalDpi);
	
	DisplayProperties::LogicalDpiChanged +=
        ref new DisplayPropertiesEventHandler(this, &MainPage::OnLogicalDpiChanged);

    Window::Current->CoreWindow->SizeChanged +=
        ref new TypedEventHandler<CoreWindow^, WindowSizeChangedEventArgs^>(this, &MainPage::OnWindowSizeChanged);

    VisualStateManager::GoToState(this, "ClearState", true);

	m_eventToken = CompositionTarget::Rendering::add(ref new EventHandler<Object^>(this, &MainPage::OnRendering));
	m_timer = ref new BasicTimer();
}

void MainPage::ApplyEffectsClicked(Object^ sender, RoutedEventArgs^ e)
{
    GeneralTransform^ t = ApplyEffectsButtonBorder->TransformToVisual(this);
    auto point = t->TransformPoint(Point(0, 0));

    EffectsPopup->VerticalOffset = point.Y - EffectsFlyout->Height - 10;
    EffectsPopup->HorizontalOffset = point.X;
    EffectsPopup->IsOpen = true;
}

void MainPage::SobelBtnClicked()
{	
	m_effects->ApplyEffects(image_state::image_state_sobel);    
}

void MainPage::PrewitBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_prewitt);
}

void MainPage::ScharBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_scharr);
}

void MainPage::EmbossBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_emboss);
}

void MainPage::GaussBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_gaussian);
}

void MainPage::SharpenBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_sharpen);
}

void MainPage::MeanRemBtnClicked()
{
	m_effects->ApplyEffects(image_state::image_state_mean_removal);
}


void MainPage::OpenClicked(Object^ sender, RoutedEventArgs^ e)
{
	FileOpenPicker^ openPicker = ref new FileOpenPicker();
	
	openPicker->FileTypeFilter->Append(".png");
	openPicker->FileTypeFilter->Append(".jpg");
	openPicker->FileTypeFilter->Append(".bmp");
	openPicker->SuggestedStartLocation = Pickers::PickerLocationId::PicturesLibrary;
	
	create_task(openPicker->PickSingleFileAsync()).then([=](StorageFile^ file)
    {
        if (file == nullptr)
        {
            cancel_current_task();
        }
		m_effects->PictureFile = file;
		TheAppBar->IsOpen = false;
		m_effects->LoadImage();
		m_renderNeeded = true;
	});
}


void MainPage::OnLogicalDpiChanged(Object^ /* sender */)
{
    m_effects->SetDpi(DisplayProperties::LogicalDpi);
}

void MainPage::OnWindowSizeChanged(CoreWindow^ /* sender */, WindowSizeChangedEventArgs^ args)
{
    if (ApplicationView::Value == ApplicationViewState::Snapped)
    {
        VisualStateManager::GoToState(this, "SnappedState", true);
    }
    else
    {
        VisualStateManager::GoToState(this, "StartingState", true);        
    }

    m_effects->UpdateForWindowSizeChange();
}


void MainPage::OnRendering(Object^ sender, Object^ args)
{
	if (m_renderNeeded)
	{
		m_timer->Update();
		m_renderNeeded = false;		
	}
}