//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

#include "pch.h"
#include "PictureEffects.h"
#include "..\DirectXTK\WICTextureLoader.h"

using namespace concurrency;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::ApplicationModel;
using namespace Windows::ApplicationModel::Activation;
using namespace Windows::ApplicationModel::Core;
using namespace Windows::ApplicationModel::DataTransfer;
using namespace Windows::Foundation;
using namespace Windows::Graphics::Display;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::UI::ApplicationSettings;
using namespace Windows::UI::Core;
using namespace Windows::UI::Input;
using namespace Windows::UI::Input::Inking;
using namespace Windows::UI::ViewManagement;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Input;
using namespace Concurrency::direct3d;
using namespace Concurrency::graphics;
using namespace Concurrency::graphics::direct3d;


PictureEffects::PictureEffects() :	
	m_loadingComplete(false)
{
	m_ampProcessedTexture = nullptr;	
}

PictureEffects::~PictureEffects()
{
	if (m_AmpComputeEngine != nullptr)
	{
		delete m_AmpComputeEngine;
		m_AmpComputeEngine = nullptr;
	}
	if (m_ampProcessedTexture != nullptr)
	{
		delete m_ampProcessedTexture;
		m_ampProcessedTexture = nullptr;
	}
	if (m_vertexShader)
		m_vertexShader.Reset();
	if (m_pixelShader)
		m_pixelShader.Reset();
	if (m_inputLayout)
		m_inputLayout.Reset();
	if (m_samplerState)
		m_samplerState.Reset();
	if (m_vertexBuffer)
		m_vertexBuffer.Reset();
	if (m_inputTexture)
		m_inputTexture.Reset();
	if (m_inputTextureSRV)
		m_inputTextureSRV.Reset();
	if (m_processedTextureSRV)
		m_processedTextureSRV.Reset();
}

void PictureEffects::CreateDeviceIndependentResources()
{
    DirectXBase::CreateDeviceIndependentResources();
}

void PictureEffects::CreateDeviceResources()
{
    DirectXBase::CreateDeviceResources();

	auto loadVSTask = DX::ReadDataAsync("vs.cso");
	auto loadPSTask = DX::ReadDataAsync("ps.cso");

	// load vshader
	auto createVSTask = loadVSTask.then([this](Platform::Array<byte>^ fileData) {
		DX::ThrowIfFailed(m_d3dDevice->CreateVertexShader(fileData->Data, fileData->Length, nullptr, &m_vertexShader));
        const D3D11_INPUT_ELEMENT_DESC basicVertexLayoutDesc[] = 
		{
			{ "POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0,  0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
			{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		};

		DX::ThrowIfFailed(m_d3dDevice->CreateInputLayout(basicVertexLayoutDesc, ARRAYSIZE(basicVertexLayoutDesc), fileData->Data, fileData->Length, &m_inputLayout)); 
		m_d3dContext->IASetInputLayout(m_inputLayout.Get());

    });

	auto createPSTask = loadPSTask.then([this](Platform::Array<byte>^ fileData){
        m_d3dDevice->CreatePixelShader( fileData->Data, fileData->Length, nullptr, &m_pixelShader );    
    });

	auto createAmpTask = (createPSTask && createVSTask).then([this] () 
	{
		//associate the DXGI Device to the C++AMP Accelerator       
		m_AmpComputeEngine = new AMP_compute_engine(m_d3dDevice.Get());
		
		// Create the sample state
		D3D11_SAMPLER_DESC sampDesc;
		ZeroMemory( &sampDesc, sizeof(sampDesc) );
		sampDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
		sampDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
		sampDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
		sampDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
		sampDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
		sampDesc.MinLOD = 0;
		sampDesc.MaxLOD = D3D11_FLOAT32_MAX;
		DX::ThrowIfFailed(m_d3dDevice->CreateSamplerState( &sampDesc, &m_samplerState ));

		// Create vertex buffer
		D3D11_BUFFER_DESC bd;
		ZeroMemory( &bd, sizeof(bd) );
		bd.Usage = D3D11_USAGE_DEFAULT;
		bd.ByteWidth = sizeof(simple_vertex) * 4;
		bd.BindFlags = D3D11_BIND_VERTEX_BUFFER;
		bd.CPUAccessFlags = 0;
		DX::ThrowIfFailed(m_d3dDevice->CreateBuffer( &bd, nullptr, &m_vertexBuffer ));

		// Set vertex buffer
		UINT stride = sizeof(simple_vertex);
		UINT offset = 0;
		std::array<ID3D11Buffer*, 1> arBuffer;
		arBuffer[0] = m_vertexBuffer.Get() ;
		m_d3dContext->IASetVertexBuffers( 0, 1, arBuffer.data(), &stride, &offset );

		// Set primitive topology
		m_d3dContext->IASetPrimitiveTopology( D3D11_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP );
	});

	createAmpTask.then([this]()
	{
		m_loadingComplete = true;
	});

}

void PictureEffects::CreateWindowSizeDependentResources()
{
    DirectXBase::CreateWindowSizeDependentResources(); 
}


void PictureEffects::UpdateForWindowSizeChange()
{
    // Release any extra references to Direct3D resources before attempting
    // to resize the swap chain.    

    DirectXBase::UpdateForWindowSizeChange();
}

void PictureEffects::LoadImage()
{
	if (m_inputTexture != nullptr)
	{
		assert(m_inputTextureSRV);
		m_inputTexture.Reset();
		m_inputTextureSRV.Reset();
	}
	
	if (m_imageFile == nullptr)
	{
		return;
	}
	auto readTask = DX::ReadPictureDataAsync(m_imageFile);
	auto createVSTask = readTask.then([this](Platform::Array<byte>^ fileData)
	{
		ComPtr<ID3D11ShaderResourceView> sptextureView;
		ComPtr<ID3D11Resource> sptexture;
		DX::ThrowIfFailed(CreateWICTextureFromMemory(m_d3dDevice.Get(), m_d3dContext.Get(), &fileData[0], (size_t)fileData->Length, &sptexture, &sptextureView, NULL)); 
		sptexture.As(&m_inputTexture);
		m_inputTextureSRV = sptextureView;

		if (m_ampProcessedTexture)
		{
			assert(m_processedTextureSRV);
			delete m_ampProcessedTexture;
			m_processedTextureSRV.Reset();
		}

		D3D11_TEXTURE2D_DESC input_tex_desc;
		m_inputTexture->GetDesc(&input_tex_desc);

		UINT imageWidth = input_tex_desc.Width;
		UINT imageHeight = input_tex_desc.Height;

		// Create a texture the same size as m_inputTexture, it's used to store the effect applied texture		
		m_ampProcessedTexture = new texture<unorm4, 2>(static_cast<int>(imageHeight), static_cast<int>(imageWidth), 8U, m_AmpComputeEngine->getView());

		// Get the D3D texture from interop, and create the corresponding SRV that could be later bound to graphics pipeline
		ID3D11Texture2D * processedTexture = reinterpret_cast<ID3D11Texture2D *>(get_texture<unorm4, 2>(*m_ampProcessedTexture));
		m_d3dDevice->CreateShaderResourceView(processedTexture, nullptr, m_processedTextureSRV.GetAddressOf());
    
		processedTexture->Release();

		float ratio = (float)imageWidth / (float)imageHeight;
	
		size_t windowWidth = static_cast<size_t>(m_window->Bounds.Width);
		size_t windowHeight = static_cast<size_t>(m_window->Bounds.Height);

		// resize the image to fit into the window without changing the ratio
		// will only scale-down
		if (imageHeight > static_cast<UINT>(windowHeight))
		{
			imageHeight = min(imageHeight, static_cast<UINT>(windowHeight));
			imageWidth = static_cast<size_t>(static_cast<float>(imageHeight) * ratio);
		}

		if (imageWidth > static_cast<UINT>(windowWidth)) 
		{
			imageWidth =  min(imageWidth, static_cast<UINT>(windowWidth));
			imageHeight = static_cast<size_t>(static_cast<float>(imageWidth) / ratio);
		}
	
		DXGI_SWAP_CHAIN_DESC1 sd;
		m_swapChain->GetDesc1(&sd);
		if (sd.Width != imageWidth || sd.Height != imageHeight) 
		{
			DX::ThrowIfFailed(m_swapChain->ResizeBuffers(sd.BufferCount, static_cast<UINT>(windowWidth), static_cast<UINT>(windowHeight), DXGI_FORMAT_R8G8B8A8_UNORM, 0)); 	
		}

		// Create a render target view using the back buffer
        ComPtr<ID3D11Texture2D> pBackBuffer = nullptr;
		DX::ThrowIfFailed(m_swapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), &pBackBuffer ));
    

		ComPtr<ID3D11RenderTargetView> pRenderTargetView = nullptr;
		DX::ThrowIfFailed(m_d3dDevice->CreateRenderTargetView(pBackBuffer.Get(), nullptr, &pRenderTargetView));
		pBackBuffer.Reset();
    
		std::array<ID3D11RenderTargetView*, 1> rgRViews = { pRenderTargetView.Get() };
		m_d3dContext->OMSetRenderTargets( UINT(rgRViews.size()), rgRViews.data(), nullptr );
	
		// Setup the viewport
		D3D11_VIEWPORT vp;
		vp.Width = static_cast<FLOAT>(windowWidth);
		vp.Height = static_cast<FLOAT>(windowHeight);
		vp.MinDepth = 0.0f;
		vp.MaxDepth = 1.0f;
		vp.TopLeftX = 0;
		vp.TopLeftY = 0;
    
		m_d3dContext->RSSetViewports( 1, &vp );

		// set the vertex buffer with four vetices that describe the shape of the 
		// resized image (without changing the aspect ratio)
		float h_offset = static_cast<float>(imageHeight) / static_cast<float>(windowHeight);
		float w_offset = static_cast<float>(imageWidth) / static_cast<float>(windowWidth);
		float d_offset = 0.1f;
		simple_vertex vertices[] =
		{
			{ float3( w_offset,  h_offset, d_offset), float2(1.0f, 0.0f) },
			{ float3( w_offset, -h_offset, d_offset), float2(1.0f, 1.0f) },
			{ float3(-w_offset,  h_offset, d_offset), float2(0.0f, 0.0f) },
			{ float3(-w_offset, -h_offset, d_offset), float2(0.0f, 1.0f) },
		
		};

		m_d3dContext->UpdateSubresource(m_vertexBuffer.Get(), 0, nullptr, vertices, 0, 0);

		// Clear the back buffer
		float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 0.0f }; // clear with black
		m_d3dContext->ClearRenderTargetView(pRenderTargetView.Get(), ClearColor);

		// Render
		m_d3dContext->VSSetShader( m_vertexShader.Get(), nullptr, 0 );
		m_d3dContext->PSSetShader( m_pixelShader.Get(), nullptr, 0 );
		if ((m_inputTextureSRV != nullptr)) 
		{
			ID3D11ShaderResourceView* rgSRVs[1];
			rgSRVs[0] = m_inputTextureSRV.Get();
			m_d3dContext->PSSetShaderResources( 0, 1, rgSRVs );
		}
		
		std::array<ID3D11SamplerState*, 1> aRState = { m_samplerState.Get() };
		m_d3dContext->PSSetSamplers( 0, UINT(aRState.size()), aRState.data() );
		m_d3dContext->Draw(4, 0); // draw the four vertices

		// Present our back buffer to our front buffer
		m_swapChain->Present(0, 0);

		// Clean the pipeline
		ID3D11ShaderResourceView * emptySRV = nullptr;
		m_d3dContext->PSSetShaderResources(0, 1, &emptySRV);
		if (pRenderTargetView) pRenderTargetView.Reset();

	});
		
}

void PictureEffects::ApplyEffects(image_state state)
{
	if (state != image_state_original && state != image_state_none)
    {
		// output texture view
        writeonly_texture_view<unorm4, 2> output_tex_view(*m_ampProcessedTexture);
        // input texture, original picture
		const texture<unorm4, 2> input_tex = make_texture<unorm4, 2>(m_AmpComputeEngine->getView(), reinterpret_cast<IUnknown *>(m_inputTexture.Get()));
        assert(input_tex.extent == output_tex_view.extent);

		switch(state)
        {
        case image_state_sobel:
            m_AmpComputeEngine->ApplyEdgeDetection<sobel_kind>(input_tex, output_tex_view);
            break;
        case image_state_prewitt:
            m_AmpComputeEngine->ApplyEdgeDetection<prewitt_kind>(input_tex, output_tex_view);
            break;
        case image_state_scharr:
            m_AmpComputeEngine->ApplyEdgeDetection<scharr_kind>(input_tex, output_tex_view);
            break;
        case image_state_emboss:
            m_AmpComputeEngine->ApplyOneConvolutionFilter<emboss_kind>(input_tex, output_tex_view);
            break;
        case image_state_gaussian:
            m_AmpComputeEngine->ApplyOneConvolutionFilter<gaussian_kind>(input_tex, output_tex_view);
            break;
        case image_state_sharpen:
            m_AmpComputeEngine->ApplyOneConvolutionFilter<sharpen_kind>(input_tex, output_tex_view);
            break;
        case image_state_mean_removal:
            m_AmpComputeEngine->ApplyOneConvolutionFilter<mean_removal_kind>(input_tex, output_tex_view);
            break;
        default:
            assert(false);
            break;
        }
		ShowProcessedImage();
	}
}

void PictureEffects::ShowProcessedImage()
{
	if (m_inputTexture == nullptr)
	{
		return;
	}

	D3D11_TEXTURE2D_DESC input_tex_desc;
	m_inputTexture->GetDesc(&input_tex_desc);

	size_t imageWidth = input_tex_desc.Width;
	size_t imageHeight = input_tex_desc.Height;

	float ratio = static_cast<float>( imageWidth ) / static_cast<float>( imageHeight );
	
	size_t windowWidth = static_cast<size_t>(m_window->Bounds.Width);
	size_t windowHeight = static_cast<size_t>(m_window->Bounds.Height);

	// resize the image to fit into the window without changing the ratio
	// will only scale-down
	if (imageHeight > windowHeight)
	{
		imageHeight = min(imageHeight, windowHeight);
		imageWidth = static_cast<size_t>(static_cast<float>(imageHeight) * ratio);
	}

	if (imageWidth > windowWidth) 
	{
		imageWidth =  min(imageWidth, windowWidth);
		imageHeight = static_cast<size_t>(static_cast<float>(imageWidth) / ratio);
	}
	
	DXGI_SWAP_CHAIN_DESC1 sd;
	m_swapChain->GetDesc1(&sd);
	if (sd.Width != imageWidth || sd.Height != imageHeight) 
	{
		DX::ThrowIfFailed(m_swapChain->ResizeBuffers(sd.BufferCount, static_cast<UINT>(windowWidth), static_cast<UINT>(windowHeight), DXGI_FORMAT_R8G8B8A8_UNORM, 0)); 	
	}

    // Create a render target view using the back buffer
    ComPtr<ID3D11Texture2D> pBackBuffer = nullptr;
	DX::ThrowIfFailed(m_swapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), &pBackBuffer ));
    

	ComPtr<ID3D11RenderTargetView> pRenderTargetView = nullptr;
	DX::ThrowIfFailed(m_d3dDevice->CreateRenderTargetView(pBackBuffer.Get(), nullptr, &pRenderTargetView));
	pBackBuffer.Reset();
    
	std::array<ID3D11RenderTargetView*, 1> rgRViews = { pRenderTargetView.Get() };
	m_d3dContext->OMSetRenderTargets( static_cast<UINT>( (rgRViews.size() )), rgRViews.data(), nullptr );
	
	// Setup the viewport
	D3D11_VIEWPORT vp;
	vp.Width = static_cast<FLOAT>(windowWidth);
	vp.Height = static_cast<FLOAT>(windowHeight);
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	vp.TopLeftX = 0;
	vp.TopLeftY = 0;
    
	m_d3dContext->RSSetViewports( 1, &vp );

	// set the vertex buffer with four vetices that describe the shape of the 
	// resized image (without changing the aspect ratio)
	float h_offset = static_cast<float>(imageHeight) / static_cast<float>(windowHeight);
	float w_offset = static_cast<float>(imageWidth) / static_cast<float>(windowWidth);
	float d_offset = 0.1f;
	simple_vertex vertices[] =
	{
		{ float3( w_offset,  h_offset, d_offset), float2(1.0f, 0.0f) },
		{ float3( w_offset, -h_offset, d_offset), float2(1.0f, 1.0f) },
		{ float3(-w_offset,  h_offset, d_offset), float2(0.0f, 0.0f) },
		{ float3(-w_offset, -h_offset, d_offset), float2(0.0f, 1.0f) },
		
	};

	m_d3dContext->UpdateSubresource(m_vertexBuffer.Get(), 0, nullptr, vertices, 0, 0);

	// Clear the back buffer
	float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 0.0f }; // clear with black
	m_d3dContext->ClearRenderTargetView(pRenderTargetView.Get(), ClearColor);

	// Render
	m_d3dContext->VSSetShader( m_vertexShader.Get(), nullptr, 0 );
	m_d3dContext->PSSetShader( m_pixelShader.Get(), nullptr, 0 );
	// use the m_processedTextureSRV
	{
		ID3D11ShaderResourceView* rgSRVs[1];
		rgSRVs[0] = m_processedTextureSRV.Get();
		m_d3dContext->PSSetShaderResources( 0, 1, rgSRVs );
	}
	std::array<ID3D11SamplerState*, 1> aRState = { m_samplerState.Get() };
	m_d3dContext->PSSetSamplers( 0, UINT(aRState.size()), aRState.data() );
	m_d3dContext->Draw(4, 0); // draw the four vertices

	// Present our back buffer to our front buffer
	m_swapChain->Present(0, 0);

	// Clean the pipeline
	ID3D11ShaderResourceView * emptySRV = nullptr;
	m_d3dContext->PSSetShaderResources(0, 1, &emptySRV);
	if (pRenderTargetView) pRenderTargetView.Reset();
}