//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

#pragma once

#include "pch.h"
#include "DirectXBase.h"
#include "ComputeEngine.h"
#include <amp.h>
#include <amprt.h>
#include <amp_graphics.h>
#include <amp_short_vectors.h>


ref class PictureEffects : public DirectXBase
{
internal:
    PictureEffects();
	

    virtual void CreateDeviceIndependentResources() override;
    virtual void CreateDeviceResources() override;
    virtual void CreateWindowSizeDependentResources() override;
    virtual void UpdateForWindowSizeChange() override;
    
    // Public saving methods.
	property Windows::Storage::StorageFile^ PictureFile
	{
		void set(Windows::Storage::StorageFile^ value)
		{
			m_imageFile = value;
		}
	}

	void LoadImage();
	void ApplyEffects(image_state state);
	void ShowProcessedImage();
private:
    
	Windows::Storage::StorageFile^							m_imageFile;
    Windows::UI::Xaml::Controls::SwapChainBackgroundPanel^  m_swapChainPanel;
    
	bool													m_loadingComplete;
	
	// D3D data members

	Microsoft::WRL::ComPtr<ID3D11VertexShader>				m_vertexShader;
	Microsoft::WRL::ComPtr<ID3D11PixelShader>				m_pixelShader;
	Microsoft::WRL::ComPtr<ID3D11InputLayout>				m_inputLayout;	
	Microsoft::WRL::ComPtr<ID3D11SamplerState>				m_samplerState;
	Microsoft::WRL::ComPtr<ID3D11Buffer>					m_vertexBuffer;
	Microsoft::WRL::ComPtr<ID3D11Texture2D>					m_inputTexture;
	Microsoft::WRL::ComPtr<ID3D11ShaderResourceView>		m_inputTextureSRV;
	Microsoft::WRL::ComPtr<ID3D11ShaderResourceView>		m_processedTextureSRV;

protected private:
	~PictureEffects();
	//C++AMP types
	AMP_compute_engine*										m_AmpComputeEngine;
	
	Concurrency::graphics::texture<Concurrency::graphics::direct3d::unorm4, 2>*		m_ampProcessedTexture;
	// for setup vertex buffer
	struct simple_vertex
	{
		Concurrency::graphics::direct3d::float3 Pos;
		Concurrency::graphics::direct3d::float2 Tex;
	};
};
