#pragma once
#include "Helpers.h"
#include <wrl\wrappers\corewrappers.h>
using namespace Microsoft::WRL::Wrappers;

namespace Demo
{
	class BaseApplicationInitializationCallback :   
		public ABI::Windows::UI::Xaml::IApplicationInitializationCallback  
	{  
	public:  
		IFACEMETHODIMP_(ULONG) AddRef()  
		{  
			return InterlockedIncrement(&m_cRef);  
		}  
  
		IFACEMETHODIMP_(ULONG) Release()  
		{  
			LONG cRef = InterlockedDecrement(&m_cRef);  
  
			if (0 == cRef)  
			{  
				delete this;  
			}  
  
			return cRef;  
		}  
  
		IFACEMETHODIMP QueryInterface(__in REFIID riid, __deref_out void** ppvObject)  
		{  
			*ppvObject = NULL;  
          
			if (IID_IUnknown == riid)  
			{  
				*ppvObject = static_cast<IUnknown*>(this);  
			}  
			else if (__uuidof(ABI::Windows::UI::Xaml::IApplicationInitializationCallback) == riid)  
			{  
				*ppvObject = static_cast<ABI::Windows::UI::Xaml::IApplicationInitializationCallback*>(this);  
			}  
			else if (IID_IMarshal == riid)  
			{  
				return m_pUnkFTM->QueryInterface(riid, ppvObject);  
			}  
			else  
			{  
				return E_NOINTERFACE;  
			}  
          
			AddRef();  
			return S_OK;  
		}  
  
		IFACEMETHODIMP Invoke(__in ABI::Windows::UI::Xaml::IApplicationInitializationCallbackParams* p) = 0;  
      
	protected:  
		BaseApplicationInitializationCallback() :  
			m_cRef(1),  
			m_pUnkFTM(NULL)  
		{  
			CoCreateFreeThreadedMarshaler(this, &m_pUnkFTM);  
		}  
  
		virtual ~BaseApplicationInitializationCallback()  
		{  
			ReleaseInterface(m_pUnkFTM);  
		}  
      
	private:  
		ULONG m_cRef;  
		IUnknown* m_pUnkFTM;  
	};  
  
	template <class T>  
	class ApplicationInitializationCallback_Instantiate : public BaseApplicationInitializationCallback  
	{  
	public:  
		static Microsoft::WRL::ComPtr<ApplicationInitializationCallback_Instantiate<T>> New()  
		{  
			ComPtr<ApplicationInitializationCallback_Instantiate<T>> spApplicationInitializationCallback;  
			spApplicationInitializationCallback.Attach(new ApplicationInitializationCallback_Instantiate<T>());  
			return spApplicationInitializationCallback;      
		}  
  
		IFACEMETHODIMP Invoke(__in ABI::Windows::UI::Xaml::IApplicationInitializationCallbackParams* p)  
		{  
			HRESULT hr = S_OK;  
          
			T* pApp = new T();  
			IFCOOM(pApp);  
			pApp->Release();  
  
		Cleanup:  
			RRETURN(hr);  
		}  
	};  
  
	template <class T>  
	class ApplicationInitializationCallback_Create : public BaseApplicationInitializationCallback  
	{  
	public:  
		static Microsoft::WRL::ComPtr<ApplicationInitializationCallback_Create<T>> New()  
		{  
			ComPtr<ApplicationInitializationCallback_Create<T>> spApplicationInitializationCallback;  
			spApplicationInitializationCallback.Attach(new ApplicationInitializationCallback_Create<T>());  
			return spApplicationInitializationCallback;      
		}  
  
		IFACEMETHODIMP Invoke(__in ABI::Windows::UI::Xaml::IApplicationInitializationCallbackParams* p)  
		{  
			HRESULT hr = S_OK;  
  
			Microsoft::WRL::ComPtr<T> spApp;  
			IFC(T::Create(&spApp));  
  
		Cleanup:  
			RRETURN(hr);  
		}  
	};  
  
	template <class T>  
	class ApplicationInitializationCallback_Make : public BaseApplicationInitializationCallback  
	{  
	public:  
		static Microsoft::WRL::ComPtr<ApplicationInitializationCallback_Make<T>> New()  
		{  
			ComPtr<ApplicationInitializationCallback_Make<T>> spApplicationInitializationCallback;  
			spApplicationInitializationCallback.Attach(new ApplicationInitializationCallback_Make<T>());  
			return spApplicationInitializationCallback;      
		}  
  
		IFACEMETHODIMP Invoke(__in ABI::Windows::UI::Xaml::IApplicationInitializationCallbackParams* p)  
		{  
			Microsoft::WRL::ComPtr<T> spApp = Make<T>();  
			return S_OK;  
		}  
	};  
  
	HRESULT StartApplication(ABI::Windows::UI::Xaml::IApplicationInitializationCallback* pCallback)  
	{  
		HRESULT hr = S_OK;  		
		Microsoft::WRL::Wrappers::RoInitializeWrapper initWinrt(RO_INIT_MULTITHREADED);		
		{  	
			HStringReference applicationAcid(RuntimeClass_Windows_UI_Xaml_Application);
			Microsoft::WRL::ComPtr<ABI::Windows::UI::Xaml::IApplicationStatics> spAppStatics;  
  
			IFC(Windows::Foundation::GetActivationFactory(applicationAcid.Get(), &spAppStatics));  
			IFC(spAppStatics->Start(pCallback));  
		}  
  
	Cleanup:  
	
		return hr;  
	}
}