//Helpers.h 
//Helper functions and macros for our basic WRL Xaml App
//Use as-is at your own discretion :-)

#pragma once
#include <wrl/client.h>
#include <wrl/implements.h>
#include <inspectable.h>
#include <windows.ui.xaml.h>
#include <windows.ui.xaml.controls.h>
#include <windows.ui.xaml.controls.primitives.h>
#include <windows.ui.xaml.markup.h>
#include <string>

typedef unsigned char                   XUINT8;
typedef unsigned short                  XUINT16;
typedef wchar_t                         XCHAR;
typedef unsigned int                    XUINT32;
typedef signed int                      XINT32;
typedef unsigned long long              XUINT64;
typedef long long                       XINT64;
typedef XUINT32                         XBOOL;
typedef unsigned long                   XDWORD;
typedef unsigned char                   XBYTE;
typedef float                           XFLOAT;
typedef double                          XDOUBLE;
typedef __success(return >= 0) long     XRESULT;

// Standard HRESULT macros
#define IFC(x) { hr = (x); if (FAILED(hr)) goto Cleanup; }
#define IFCOOM(x) if ((x) == NULL) {hr = E_OUTOFMEMORY; goto Cleanup;}
#define IFCPTR(x) if ((x) == NULL) {hr = E_POINTER; goto Cleanup;}
#define IFCW32(x) if ((x) == FALSE) {hr = HRESULT_FROM_WIN32(GetLastError()); goto Cleanup;}
#define IFCHNDL(x) if ((x) == NULL) {hr = E_HANDLE; goto Cleanup;}
#define IFCEXPECT(x) if (!(x)) {hr = E_UNEXPECTED; goto Cleanup;}
#define IFCPTRRC(x,y) if ((x) == NULL) {hr = y; goto Cleanup;}
#define IGNOREHR(x) (void) (x)
#define ReleaseInterface(x) if ((x) != NULL) { (x)->Release(); x = NULL; }
#define AddRefInterface(x)  if ((x)) {(x)->AddRef();}
#define RRETURN(hr) return (hr)

using namespace ABI::Windows::UI::Xaml::Controls;
using namespace ABI::Windows::UI::Xaml::Markup;
using namespace ABI::Windows::Foundation;
using namespace ABI::Windows::Foundation::Collections;
using namespace Microsoft::WRL;

HSTRING CreateHString(__in const XCHAR* pszString);

namespace Demo
{
	//Activation helper function for WinRT object activation
	template<class T>
	class MyActivator
	{
	private:
		MyActivator()
		{
		}

	public:
		static __checkReturn XRESULT Activate(__in_z XCHAR* pszName, __deref_out T** pInstance)
		{
			XRESULT hr = S_OK; 
            HSTRING hsName = NULL;
            ComPtr<IInspectable> pInstance;
            ComPtr<T> pT;

            hsName = CreateHString(pszName);
            IFC(Windows::Foundation::ActivateInstance(hsName, &pInstance)); 
            IFC(pInstance.As<T>(&pT));
            *ppInstance = pT.Get();

Cleanup:
            pInstance.Detach();
            if (hsName) { WindowsDeleteString(hsName); }
            RRETURN(hr);
		}

		static __checkReturn XRESULT ActivateFactory(__in_z const XCHAR* pszName, __deref_out T** ppInstance)
		{
			XRESULT hr = S_OK;
            HSTRING hsName = NULL;
            ComPtr<IActivationFactory> pActivationFactory;
            ComPtr<T> pInstance;

            hsName = CreateHString(pszName);
            IFC(Windows::Foundation::GetActivationFactory(hsName, &pActivationFactory));
            IFC(pActivationFactory.As<T>(&pInstance));
            *ppInstance = pInstance.Get();
            pInstance.Detach();

Cleanup:
            if (hsName) { WindowsDeleteString(hsName); }
            RRETURN(hr);
		}
	};

	template<class T, class S>
	class MyAggregator : public IInspectable
	{
	public:
		// after creation an inner object holds the reference to an outer object 
        // We should either release m_pInnerInstance by delegating release to ourselves or initialize the counter to 0
		MyAggregator (XCHAR* pszName) :m_nRef(1), m_pInnerInspectable(NULL), m_pInnerInstance (NULL)
        {
            HRESULT hr = S_OK;
            ComPtr<S> pFactory;

            MyActivator<S>::ActivateFactory(pszName, &pFactory);
            pFactory->CreateInstance(this, &m_pInnerInspectable, &m_pInnerInstance);
            Release();
        }

        virtual ~MyAggregator()
        {
        }

		virtual STDMETHODIMP QueryInterface(REFIID iid, void **ppInterface)
        {
            //Derived class should overide this to also include any new additional interfaces that it may implement.
            if (iid == IID_IUnknown)
            {
                *ppInterface = static_cast<IUnknown *>(this);
            }
            else if (iid == IID_IInspectable)
            {
                *ppInterface = static_cast<IInspectable *>(this);
            }
            else
            {
                //Non-delegating inner QI
                return m_pInnerInspectable.Get()->QueryInterface(iid, ppInterface);
            }

            AddRef();
            RRETURN(S_OK);
        }

		STDMETHODIMP_(ULONG) AddRef()
        {
            return ++m_nRef;
        }

        STDMETHODIMP_(ULONG) Release()
        {
            if (--m_nRef == 0)
            {
                delete this;
                return 0;
            }
            return m_nRef;
        }

		virtual STDMETHODIMP GetRuntimeClassName(__out HSTRING *pClassName)
        {
            //This method should be overriden by a derived class.
            *pClassName = CreateHString(L"MyAggregator");
            return S_OK;
        }

        virtual STDMETHODIMP GetTrustLevel(__out TrustLevel *trustLvl)
        {
            return m_pInnerInspectable.Get()->GetTrustLevel(trustLvl);
        }

        virtual STDMETHODIMP GetIids(__out ULONG *iidCount, __deref_out IID **iids)
        {
            //This should be overriden by a derived class to include the extra new interfaces that it may 
            // implement.
            return m_pInnerInspectable.Get()->GetIids(iidCount, iids);
        }

    protected:
        ComPtr<IInspectable> m_pInnerInspectable;
        ComPtr<T> m_pInnerInstance;

    private:
        ULONG m_nRef;

	};

	

}