
#include "AppHelper.h"

using namespace ABI::Windows::Foundation;
using namespace Microsoft::WRL;
using namespace Microsoft::WRL::Wrappers;
using namespace ABI::Windows::UI::Xaml;
using namespace ABI::Windows::UI::Xaml::Markup;
using namespace Demo;


//create the UI
HRESULT CreateXamlUI()
{
	HRESULT hr = S_OK;
	wchar_t* pszXaml = 
		L"<StackPanel xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation' x:Name='root' "
		L"xmlns:x='http://schemas.microsoft.com/winfx/2006/xaml'> "
		L"<TextBlock Text='Hello World!' x:Name='myTextBlock' FontSize='65' /> " 		
		L"</StackPanel>";  

	HStringReference xamlAcid(RuntimeClass_Windows_UI_Xaml_Markup_XamlReader);
	HStringReference windowAcid(RuntimeClass_Windows_UI_Xaml_Window);
	

	ComPtr<IXamlReaderStatics> spXamlReader;
	HString xaml;
	ComPtr<IDependencyObject> spVisualRoot;
	ComPtr<IUIElement> spRootElementasUIElement;
	ComPtr<IWindowStatics> spWindowStatics;
	ComPtr<IWindow> spWindow;

	//Activate the Markup XAML Reader component
	IFC(GetActivationFactory(xamlAcid.Get(), &spXamlReader));
	IFC(xaml.Set(pszXaml, (UINT32) wcslen(pszXaml)));
	IFC(spXamlReader->Load(xaml.Get(), &spVisualRoot));
	IFC(spVisualRoot.As(&spRootElementasUIElement));
	
	IFC(GetActivationFactory(windowAcid.Get(), &spWindowStatics));
	IFC(spWindowStatics->get_Current(&spWindow));
	IFC(spWindow->put_Content(spRootElementasUIElement.Get()));
	IFC(spWindow->Activate());


Cleanup:
	return hr;
}

namespace Demo
{
	class HelloWorldApp : public ABI::Windows::UI::Xaml::IApplicationOverrides 
	{
	private:
		ULONG m_cRef;
		IInspectable* m_pInner;

	protected:
		HelloWorldApp() :
			m_cRef(1),
			m_pInner(nullptr)
		{
		}

		~HelloWorldApp()
		{
			ReleaseInterface(m_pInner);
		}

		void SetInnerInspectable(__in IInspectable* pInner)
		{
			AddRefInterface(pInner);
			ReleaseInterface(m_pInner);
			m_pInner = pInner;
		}

	public:
		//IUnknown  	
		IFACEMETHODIMP_(ULONG) AddRef()  
		{  
			return ++m_cRef;  
		}  
  
		IFACEMETHODIMP_(ULONG) Release()  
		{  
			ULONG cRef = --m_cRef;  
			if (0 == cRef)  
			{  
				delete this;  
			}  
			return cRef;  
		}  
  
		IFACEMETHODIMP QueryInterface(__in REFIID riid, __deref_out void** ppvObject)  
		{  
			*ppvObject = NULL;  
			if (IID_IInspectable == riid)  
			{  
				*ppvObject = static_cast<IInspectable*>(this);  
			}  
			else if (IID_IUnknown == riid)  
			{  
				*ppvObject = static_cast<IUnknown*>(this);  
			}  
			else if (__uuidof(ABI::Windows::UI::Xaml::IApplicationOverrides) == riid)  
			{  
				*ppvObject = static_cast<ABI::Windows::UI::Xaml::IApplicationOverrides*>(this);  
			}  
			else  
			{  				            
				if(m_pInner)   
				   return m_pInner->QueryInterface(riid, ppvObject);  
				else   
					return E_NOINTERFACE;  
			}  
  
			AddRef();  
			return S_OK;  
		}  
		
		//IInspectable      
		IFACEMETHODIMP GetRuntimeClassName(__out HSTRING* pRuntimeClassName)  
		{  
			return E_NOTIMPL;  
		}  
  
		IFACEMETHODIMP GetTrustLevel(__out TrustLevel* trustLevel)  
		{  
			return E_NOTIMPL;  
		}  
      
		IFACEMETHODIMP GetIids(__out ULONG* iidCount, __deref_out IID** iids)  
		{  
			return E_NOTIMPL;  
		}  

		//HelloWorldApp methods
		static HRESULT Create(__deref_out HelloWorldApp** ppApp)
		{
			HRESULT hr = S_OK;
			ComPtr<HelloWorldApp> spApp;
			HStringReference applicationAcid(RuntimeClass_Windows_UI_Xaml_Application);
			ComPtr<IApplicationFactory> spAppFactory;
			ComPtr<IInspectable> spAppInner;
			ComPtr<IApplication> spAppInnerInstance;

			spApp.Attach(new HelloWorldApp());
			IFCOOM(spApp.Get());

			IFC(GetActivationFactory(applicationAcid.Get(), &spAppFactory));
			IFC(spAppFactory->CreateInstance(spApp.Get(), &spAppInner, &spAppInnerInstance));
			spApp->SetInnerInspectable(spAppInner.Get());

			*ppApp = spApp.Detach();

	Cleanup:
			return hr;
		}

		 virtual HRESULT STDMETHODCALLTYPE OnInitialize( void)  
		{  
			return S_OK;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::IActivatedEventArgs *args)  
		{  
			return S_OK;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnLaunched(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::ILaunchActivatedEventArgs *args)  
		{  
			HRESULT hr = S_OK;  
			IFC(CreateXamlUI());		  
		Cleanup:  
			return hr;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnFileActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::IFileActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnSearchActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::ISearchActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnShareTargetActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::IShareTargetActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnFileOpenPickerActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::IFileOpenPickerActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnFileSavePickerActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::IFileSavePickerActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
                      
		virtual HRESULT STDMETHODCALLTYPE OnCachedFileUpdaterActivated(   
			/* [in] */ __RPC__in_opt ABI::Windows::ApplicationModel::Activation::ICachedFileUpdaterActivatedEventArgs *args)  
		{  
			return E_NOTIMPL;  
		}  
  
		virtual HRESULT STDMETHODCALLTYPE OnWindowCreated(   
			/* [in] */ __RPC__in_opt ABI::Windows::UI::Xaml::IWindowCreatedEventArgs *args)   
		{   
			return S_OK;   
		}  

	};
}

// Application entry point  
int APIENTRY WinMain(  
    HINSTANCE hInstance,  
    HINSTANCE hPrevInstance,  
    LPTSTR lpCmdLine,  
    int nCmdShow)  
{ 	
	
    HRESULT hr = StartApplication(  
		ApplicationInitializationCallback_Create<HelloWorldApp>::New().Get());  
    return SUCCEEDED(hr) ? 0 : -1;  
} 